@@ -5,7 +5,8 @@
 # Conditional formatting allows you to apply a format to a cell or a
 # range of cells based on certain criteria.
 #
-# Copyright 2013-2014, John McNamara, jmcnamara@cpan.org
+# SPDX-License-Identifier: BSD-2-Clause
+# Copyright 2013-2022, John McNamara, jmcnamara@cpan.org
 #
 import xlsxwriter
 
@@ -18,6 +19,7 @@ worksheet5 = workbook.add_worksheet()
 worksheet6 = workbook.add_worksheet()
 worksheet7 = workbook.add_worksheet()
 worksheet8 = workbook.add_worksheet()
+worksheet9 = workbook.add_worksheet()
 
 # Add a format. Light red fill with dark red text.
 format1 = workbook.add_format({'bg_color': '#FFC7CE',
@@ -186,56 +188,131 @@ worksheet6.conditional_format('B3:K6', {'type': 'cell',
 #
 # Example 7.
 #
-caption = 'Examples of color scales and data bars. Default colours.'
+caption = 'Examples of color scales with default and user colors.'
 
 data = range(1, 13)
 
 worksheet7.write('A1', caption)
 
 worksheet7.write('B2', "2 Color Scale")
-worksheet7.write('D2', "3 Color Scale")
-worksheet7.write('F2', "Data Bars")
+worksheet7.write('D2', "2 Color Scale + user colors")
+
+worksheet7.write('G2', "3 Color Scale")
+worksheet7.write('I2', "3 Color Scale + user colors")
 
 for row, row_data in enumerate(data):
     worksheet7.write(row + 2, 1, row_data)
     worksheet7.write(row + 2, 3, row_data)
-    worksheet7.write(row + 2, 5, row_data)
+    worksheet7.write(row + 2, 6, row_data)
+    worksheet7.write(row + 2, 8, row_data)
 
 worksheet7.conditional_format('B3:B14', {'type': '2_color_scale'})
-worksheet7.conditional_format('D3:D14', {'type': '3_color_scale'})
-worksheet7.conditional_format('F3:F14', {'type': 'data_bar'})
+
+worksheet7.conditional_format('D3:D14', {'type': '2_color_scale',
+                                         'min_color': "#FF0000",
+                                         'max_color': "#00FF00"})
+
+worksheet7.conditional_format('G3:G14', {'type': '3_color_scale'})
+
+worksheet7.conditional_format('I3:I14', {'type': '3_color_scale',
+                                         'min_color': "#C5D9F1",
+                                         'mid_color': "#8DB4E3",
+                                         'max_color': "#538ED5"})
 
 
 ###############################################################################
 #
 # Example 8.
 #
-caption = 'Examples of color scales and data bars. Modified colours.'
-
-data = range(1, 13)
-
+caption = 'Examples of data bars.'
 
 worksheet8.write('A1', caption)
 
-worksheet8.write('B2', "2 Color Scale")
-worksheet8.write('D2', "3 Color Scale")
-worksheet8.write('F2', "Data Bars")
+worksheet8.write('B2', "Default data bars")
+worksheet8.write('D2', "Bars only")
+worksheet8.write('F2', "With user color")
+worksheet8.write('H2', "Solid bars")
+worksheet8.write('J2', "Right to left")
+worksheet8.write('L2', "Excel 2010 style")
+worksheet8.write('N2', "Negative same as positive")
 
+data = range(1, 13)
 for row, row_data in enumerate(data):
     worksheet8.write(row + 2, 1, row_data)
     worksheet8.write(row + 2, 3, row_data)
     worksheet8.write(row + 2, 5, row_data)
+    worksheet8.write(row + 2, 7, row_data)
+    worksheet8.write(row + 2, 9, row_data)
 
-worksheet8.conditional_format('B3:B14', {'type': '2_color_scale',
-                                         'min_color': "#FF0000",
-                                         'max_color': "#00FF00"})
+data = [-1, -2, -3, -2, -1, 0, 1, 2, 3, 2, 1, 0]
+for row, row_data in enumerate(data):
+    worksheet8.write(row + 2, 11, row_data)
+    worksheet8.write(row + 2, 13, row_data)
 
-worksheet8.conditional_format('D3:D14', {'type': '3_color_scale',
-                                         'min_color': "#C5D9F1",
-                                         'mid_color': "#8DB4E3",
-                                         'max_color': "#538ED5"})
+worksheet8.conditional_format('B3:B14', {'type': 'data_bar'})
+
+worksheet8.conditional_format('D3:D14', {'type': 'data_bar',
+                                         'bar_only': True})
 
 worksheet8.conditional_format('F3:F14', {'type': 'data_bar',
                                          'bar_color': '#63C384'})
 
+worksheet8.conditional_format('H3:H14', {'type': 'data_bar',
+                                         'bar_solid': True})
+
+worksheet8.conditional_format('J3:J14', {'type': 'data_bar',
+                                         'bar_direction': 'right'})
+
+worksheet8.conditional_format('L3:L14', {'type': 'data_bar',
+                                         'data_bar_2010': True})
+
+worksheet8.conditional_format('N3:N14', {'type': 'data_bar',
+                                         'bar_negative_color_same': True,
+                                         'bar_negative_border_color_same': True})
+
+
+###############################################################################
+#
+# Example 9.
+#
+caption = 'Examples of conditional formats with icon sets.'
+
+data = [
+    [1, 2, 3],
+    [1, 2, 3],
+    [1, 2, 3],
+    [1, 2, 3],
+    [1, 2, 3, 4],
+    [1, 2, 3, 4, 5],
+    [1, 2, 3, 4, 5],
+]
+
+worksheet9.write('A1', caption)
+
+for row, row_data in enumerate(data):
+    worksheet9.write_row(row + 2, 1, row_data)
+
+worksheet9.conditional_format('B3:D3', {'type': 'icon_set',
+                                        'icon_style': '3_traffic_lights'})
+
+worksheet9.conditional_format('B4:D4', {'type': 'icon_set',
+                                        'icon_style': '3_traffic_lights',
+                                        'reverse_icons': True})
+
+worksheet9.conditional_format('B5:D5', {'type': 'icon_set',
+                                        'icon_style': '3_traffic_lights',
+                                        'icons_only': True})
+
+worksheet9.conditional_format('B6:D6', {'type': 'icon_set',
+                                        'icon_style': '3_arrows'})
+
+worksheet9.conditional_format('B7:E7', {'type': 'icon_set',
+                                        'icon_style': '4_arrows'})
+
+worksheet9.conditional_format('B8:F8', {'type': 'icon_set',
+                                        'icon_style': '5_arrows'})
+
+worksheet9.conditional_format('B9:F9', {'type': 'icon_set',
+                                        'icon_style': '5_ratings'})
+
 workbook.close()
