@@ -3,7 +3,8 @@
 # An example of converting a Pandas dataframe to an xlsx file with a chart
 # using Pandas and XlsxWriter.
 #
-# Copyright 2013-2019, John McNamara, jmcnamara@cpan.org
+# SPDX-License-Identifier: BSD-2-Clause
+# Copyright 2013-2022, John McNamara, jmcnamara@cpan.org
 #
 
 import pandas as pd
@@ -25,11 +26,14 @@ worksheet = writer.sheets['Sheet1']
 # Create a chart object.
 chart = workbook.add_chart({'type': 'column'})
 
+# Get the dimensions of the dataframe.
+(max_row, max_col) = df.shape
+
 # Configure the series of the chart from the dataframe data.
-chart.add_series({'values': '=Sheet1!$B$2:$B$8'})
+chart.add_series({'values': ['Sheet1', 1, 1, max_row, 1]})
 
 # Insert the chart into the worksheet.
-worksheet.insert_chart('D2', chart)
+worksheet.insert_chart(1, 3, chart)
 
 # Close the Pandas Excel writer and output the Excel file.
-writer.save()
+writer.close()
