@@ -3,7 +3,8 @@
 # An example of converting a Pandas dataframe to an xlsx file with a
 # conditional formatting using Pandas and XlsxWriter.
 #
-# Copyright 2013-2019, John McNamara, jmcnamara@cpan.org
+# SPDX-License-Identifier: BSD-2-Clause
+# Copyright 2013-2022, John McNamara, jmcnamara@cpan.org
 #
 
 import pandas as pd
@@ -22,8 +23,12 @@ df.to_excel(writer, sheet_name='Sheet1')
 workbook  = writer.book
 worksheet = writer.sheets['Sheet1']
 
-# Apply a conditional format to the cell range.
-worksheet.conditional_format('B2:B8', {'type': '3_color_scale'})
+# Get the dimensions of the dataframe.
+(max_row, max_col) = df.shape
+
+# Apply a conditional format to the required cell range.
+worksheet.conditional_format(1, max_col, max_row, max_col,
+                             {'type': '3_color_scale'})
 
 # Close the Pandas Excel writer and output the Excel file.
-writer.save()
+writer.close()
