@@ -3,7 +3,8 @@
 # An example of positioning dataframes in a worksheet using Pandas and
 # XlsxWriter.
 #
-# Copyright 2013-2019, John McNamara, jmcnamara@cpan.org
+# SPDX-License-Identifier: BSD-2-Clause
+# Copyright 2013-2022, John McNamara, jmcnamara@cpan.org
 #
 
 import pandas as pd
@@ -28,4 +29,4 @@ df4.to_excel(writer, sheet_name='Sheet1',
              startrow=7, startcol=4, header=False, index=False)
 
 # Close the Pandas Excel writer and output the Excel file.
-writer.save()
+writer.close()
