@@ -2,7 +2,8 @@
 #
 # SharedStrings - A class for writing the Excel XLSX sharedStrings file.
 #
-# Copyright 2013, John McNamara, jmcnamara@cpan.org
+# SPDX-License-Identifier: BSD-2-Clause
+# Copyright 2013-2022, John McNamara, jmcnamara@cpan.org
 #
 
 # Standard packages.
@@ -10,6 +11,11 @@ import re
 
 # Package imports.
 from . import xmlwriter
+from .utility import preserve_whitespace
+
+# Compile performance critical regular expressions.
+re_control_chars_1 = re.compile('(_x[0-9a-fA-F]{4}_)')
+re_control_chars_2 = re.compile(r'([\x00-\x08\x0b-\x1f])')
 
 
 class SharedStrings(xmlwriter.XMLwriter):
@@ -79,31 +85,35 @@ class SharedStrings(xmlwriter.XMLwriter):
     def _write_sst_strings(self):
         # Write the sst string elements.
 
-        for string in (self.string_table._get_strings()):
+        for string in (self.string_table.string_array):
             self._write_si(string)
 
     def _write_si(self, string):
         # Write the <si> element.
         attributes = []
 
-        # TODO: Fix control char encoding when unit test is ported.
         # Excel escapes control characters with _xHHHH_ and also escapes any
         # literal strings of that type by encoding the leading underscore.
         # So "\0" -> _x0000_ and "_x0000_" -> _x005F_x0000_.
         # The following substitutions deal with those cases.
 
         # Escape the escape.
-        # string =~ s/(_x[0-9a-fA-F]{4}_)/_x005F1/g
+        string = re_control_chars_1.sub(r'_x005F\1', string)
 
         # Convert control character to the _xHHHH_ escape.
-        # string =~ s/([\x00-\x08\x0B-\x1F])/sprintf "_x04X_", ord(1)/eg
+        string = re_control_chars_2.sub(lambda match: "_x%04X_" %
+                                        ord(match.group(1)), string)
+
+        # Escapes non characters in strings.
+        string = string.replace('\uFFFE', '_xFFFE_')
+        string = string.replace('\uFFFF', '_xFFFF_')
 
         # Add attribute to preserve leading or trailing whitespace.
-        if re.search('^\s', string) or re.search('\s$', string):
+        if preserve_whitespace(string):
             attributes.append(('xml:space', 'preserve'))
 
         # Write any rich strings without further tags.
-        if re.search('^<r>', string) and re.search('</r>$', string):
+        if string.startswith('<r>') and string.endswith('</r>'):
             self._xml_rich_si_element(string)
         else:
             self._xml_si_element(string, attributes)
@@ -146,7 +156,3 @@ class SharedStringTable(object):
         self.string_array = sorted(self.string_table,
                                    key=self.string_table.__getitem__)
         self.string_table = {}
-
-    def _get_strings(self):
-        """" Return the sorted string list. """
-        return self.string_array
