@@ -2,14 +2,19 @@
 #
 # Worksheet - A class for writing Excel Worksheets.
 #
-# Copyright 2013-2015, John McNamara, jmcnamara@cpan.org
+# SPDX-License-Identifier: BSD-2-Clause
+# Copyright 2013-2022, John McNamara, jmcnamara@cpan.org
 #
 import re
 import datetime
 from warnings import warn
 
 COL_NAMES = {}
-range_parts = re.compile(r'(\$?)([A-Z]{1,3})(\$?)(\d+)')
+
+# Compile performance critical regular expressions.
+re_leading = re.compile(r'^\s')
+re_trailing = re.compile(r'\s$')
+re_range_parts = re.compile(r'(\$?)([A-Z]{1,3})(\$?)(\d+)')
 
 
 def xl_rowcol_to_cell(row, col, row_abs=False, col_abs=False):
@@ -26,6 +31,14 @@ def xl_rowcol_to_cell(row, col, row_abs=False, col_abs=False):
         A1 style string.
 
     """
+    if row < 0:
+        warn("Row number %d must be >= 0" % row)
+        return None
+
+    if col < 0:
+        warn("Col number %d must be >= 0" % col)
+        return None
+
     row += 1  # Change to 1-index.
     row_abs = '$' if row_abs else ''
 
@@ -36,7 +49,7 @@ def xl_rowcol_to_cell(row, col, row_abs=False, col_abs=False):
 
 def xl_rowcol_to_cell_fast(row, col):
     """
-    Optimised version of the xl_rowcol_to_cell function. Only used internally.
+    Optimized version of the xl_rowcol_to_cell function. Only used internally.
 
     Args:
        row: The cell row.    Int.
@@ -55,7 +68,7 @@ def xl_rowcol_to_cell_fast(row, col):
     return col_str + str(row + 1)
 
 
-def xl_col_to_name(col_num, col_abs=False):
+def xl_col_to_name(col, col_abs=False):
     """
     Convert a zero indexed column cell reference to a string.
 
@@ -67,6 +80,11 @@ def xl_col_to_name(col_num, col_abs=False):
         Column style string.
 
     """
+    col_num = col
+    if col_num < 0:
+        warn("Col number %d must be >= 0" % col_num)
+        return None
+
     col_num += 1  # Change to 1-index.
     col_str = ''
     col_abs = '$' if col_abs else ''
@@ -104,7 +122,7 @@ def xl_cell_to_rowcol(cell_str):
     if not cell_str:
         return 0, 0
 
-    match = range_parts.match(cell_str)
+    match = re_range_parts.match(cell_str)
     col_str = match.group(2)
     row_str = match.group(4)
 
@@ -137,7 +155,7 @@ def xl_cell_to_rowcol_abs(cell_str):
     if not cell_str:
         return 0, 0, False, False
 
-    match = range_parts.match(cell_str)
+    match = re_range_parts.match(cell_str)
 
     col_abs = match.group(1)
     col_str = match.group(2)
@@ -185,7 +203,14 @@ def xl_range(first_row, first_col, last_row, last_col):
     range1 = xl_rowcol_to_cell(first_row, first_col)
     range2 = xl_rowcol_to_cell(last_row, last_col)
 
-    return range1 + ':' + range2
+    if range1 is None or range2 is None:
+        warn("Row and column numbers must be >= 0")
+        return None
+
+    if range1 == range2:
+        return range1
+    else:
+        return range1 + ':' + range2
 
 
 def xl_range_abs(first_row, first_col, last_row, last_col):
@@ -206,7 +231,14 @@ def xl_range_abs(first_row, first_col, last_row, last_col):
     range1 = xl_rowcol_to_cell(first_row, first_col, True, True)
     range2 = xl_rowcol_to_cell(last_row, last_col, True, True)
 
-    return range1 + ':' + range2
+    if range1 is None or range2 is None:
+        warn("Row and column numbers must be >= 0")
+        return None
+
+    if range1 == range2:
+        return range1
+    else:
+        return range1 + ':' + range2
 
 
 def xl_range_formula(sheetname, first_row, first_col, last_row, last_col):
@@ -249,7 +281,7 @@ def quote_sheetname(sheetname):
         # Double quote any single quotes.
         sheetname = sheetname.replace("'", "''")
 
-        # Singe quote the sheet name.
+        # Single quote the sheet name.
         sheetname = "'%s'" % sheetname
 
     return sheetname
@@ -257,7 +289,7 @@ def quote_sheetname(sheetname):
 
 def xl_color(color):
     # Used in conjunction with the XlsxWriter *color() methods to convert
-    # a colour name into an RGB formatted string. These colours are for
+    # a color name into an RGB formatted string. These colors are for
     # backward compatibility with older versions of Excel.
     named_colors = {
         'black': '#000000',
@@ -288,6 +320,16 @@ def xl_color(color):
     return "FF" + color.lstrip('#').upper()
 
 
+def get_rgb_color(color):
+    # Convert the user specified color to an RGB color.
+    rgb_color = xl_color(color)
+
+    # Remove leading FF from RGB color for charts.
+    rgb_color = re.sub(r'^FF', '', rgb_color)
+
+    return rgb_color
+
+
 def get_sparkline_style(style_id):
     styles = [
         {'series':   {'theme': "4", 'tint': "-0.499984740745262"},
@@ -593,16 +635,34 @@ def get_sparkline_style(style_id):
 
 def supported_datetime(dt):
     # Determine is an argument is a supported datetime object.
-    return(isinstance(dt, (datetime.datetime,
+    return isinstance(dt, (datetime.datetime,
                            datetime.date,
                            datetime.time,
-                           datetime.timedelta)))
+                           datetime.timedelta))
 
 
-def datetime_to_excel_datetime(dt_obj, date_1904):
+def remove_datetime_timezone(dt_obj, remove_timezone):
+    # Excel doesn't support timezones in datetimes/times so we remove the
+    # tzinfo from the object if the user has specified that option in the
+    # constructor.
+    if remove_timezone:
+        dt_obj = dt_obj.replace(tzinfo=None)
+    else:
+        if dt_obj.tzinfo:
+            raise TypeError(
+                "Excel doesn't support timezones in datetimes. "
+                "Set the tzinfo in the datetime/time object to None or "
+                "use the 'remove_timezone' Workbook() option")
+
+    return dt_obj
+
+
+def datetime_to_excel_datetime(dt_obj, date_1904, remove_timezone):
     # Convert a datetime object to an Excel serial date and time. The integer
     # part of the number stores the number of days since the epoch and the
     # fractional part stores the percentage of the day.
+    date_type = dt_obj
+    is_timedelta = False
 
     if date_1904:
         # Excel for Mac date epoch.
@@ -614,14 +674,17 @@ def datetime_to_excel_datetime(dt_obj, date_1904):
     # We handle datetime .datetime, .date and .time objects but convert
     # them to datetime.datetime objects and process them in the same way.
     if isinstance(dt_obj, datetime.datetime):
+        dt_obj = remove_datetime_timezone(dt_obj, remove_timezone)
         delta = dt_obj - epoch
     elif isinstance(dt_obj, datetime.date):
         dt_obj = datetime.datetime.fromordinal(dt_obj.toordinal())
         delta = dt_obj - epoch
     elif isinstance(dt_obj, datetime.time):
         dt_obj = datetime.datetime.combine(epoch, dt_obj)
+        dt_obj = remove_datetime_timezone(dt_obj, remove_timezone)
         delta = dt_obj - epoch
     elif isinstance(dt_obj, datetime.timedelta):
+        is_timedelta = True
         delta = dt_obj
     else:
         raise TypeError("Unknown or unsupported datetime type")
@@ -632,14 +695,24 @@ def datetime_to_excel_datetime(dt_obj, date_1904):
                      + float(delta.microseconds) / 1E6)
                   / (60 * 60 * 24))
 
-    # Special case for datetime where time only has been specified and
-    # the default date of 1900-01-01 is used.
-    if (not isinstance(dt_obj, datetime.timedelta)
+    # The following is a workaround for the fact that in Excel a time only
+    # value is represented as 1899-12-31+time whereas in datetime.datetime()
+    # it is 1900-1-1+time so we need to subtract the 1 day difference.
+    if (isinstance(date_type, datetime.datetime)
             and dt_obj.isocalendar() == (1900, 1, 1)):
         excel_time -= 1
 
     # Account for Excel erroneously treating 1900 as a leap year.
-    if not date_1904 and excel_time > 59:
+    if not date_1904 and not is_timedelta and excel_time > 59:
         excel_time += 1
 
     return excel_time
+
+
+def preserve_whitespace(string):
+    # Check if a string has leading or trailing whitespace that requires a
+    # "preserve" attribute.
+    if (re_leading.search(string) or re_trailing.search(string)):
+        return True
+    else:
+        return False
